@@ -104,6 +104,16 @@ using the configure script.
 
 #endif
 
+#if 0
+#define NTL_THREAD_BOOST
+
+/* Set if you want to compile NTL to exploit threads internally.
+ *
+ * To re-build after changing this flag: rm *.o; make ntl.a
+ */
+
+#endif
+#
 
 #if 1
 #define NTL_GMP_LIP
@@ -148,19 +158,6 @@ using the configure script.
 
 #endif
 
-#if 0
-#define NTL_PCLMUL
-
-/* 
- * Use this flag for faster GF2X arithmetc.  
- * This enables the use of the PCLMUL instruction on x86-64
- * machines. 
- *
- * To re-build after changing this flag:
- *   rm GF2X.o; make ntl.a
- */
-
-#endif
 
 #if 0
 #define NTL_LONG_LONG_TYPE long long
@@ -301,8 +298,7 @@ using the configure script.
 #if 0
 #define NTL_DISABLE_LONGDOUBLE
 
-/* Explicitly disables us of long double arithmetic in the 
- * single-precision modular arithmetic routines
+/* Explicitly disables us of long double arithmetic
  */
 
 #endif
@@ -311,13 +307,22 @@ using the configure script.
 #if 0
 #define NTL_DISABLE_LONGLONG
 
-/* Explicitly disables us of long long arithmetic in the 
- * single-precision modular arithmetic routines
+/* Explicitly disables us of long long arithmetic 
  */
 
 #endif
 
 
+#if 0
+#define NTL_MAXIMIZE_SP_NBITS
+
+/* Allows for 62-bit single-precision moduli on 64-bit platforms.
+ * By default, such moduli are restricted to 60 bits, which
+ * usually gives slightly better performance across a range of
+ * of parameters.
+ */
+
+#endif
 
 /*************************************************************************
  *
@@ -506,6 +511,34 @@ using the configure script.
 #endif
 
 
+#if 0
+#define NTL_CRT_ALTCODE
+
+/*
+ * Employs an alternative CRT strategy.
+ * Only relevant with GMP.
+ * Seems to be marginally faster on some x86_64 platforms.
+ *
+ *   To re-build after changing this flag: 
+ *      rm lip.o; make ntl.a
+ */
+
+#endif
+
+#if 1
+#define NTL_CRT_ALTCODE_SMALL
+
+/*
+ * Employs an alternative CRT strategy for small moduli.
+ * Only relevant with GMP.
+ * Seems to be marginally faster on some x86_64 platforms.
+ *
+ *   To re-build after changing this flag: 
+ *      rm lip.o; make ntl.a
+ */
+
+#endif
+
 
 #if 0
 #define NTL_GF2X_ALTCODE
@@ -548,6 +581,19 @@ using the configure script.
 #endif
 
 
+#if 0
+#define NTL_PCLMUL
+
+/* 
+ * Use this flag for faster GF2X arithmetc.  
+ * This enables the use of the PCLMUL instruction on x86-64
+ * machines. 
+ *
+ * To re-build after changing this flag:
+ *   rm GF2X.o; make ntl.a
+ */
+
+#endif
 
 
 
