@@ -4,8 +4,10 @@
 
 #include <NTL/config.h>
 #include <NTL/mach_desc.h>
-#include <NTL/have_LL.h>
-#include <NTL/have_builtin_clzl.h>
+#include <NTL/HAVE_LL_TYPE.h>
+#include <NTL/HAVE_BUILTIN_CLZL.h>
+#include <NTL/HAVE_AVX.h>
+#include <NTL/HAVE_FMA.h>
 
 
 /*
@@ -72,11 +74,53 @@
 
 #define NTL_ULL_TYPE unsigned long long
 
+#endif
+
+
+#ifdef NTL_HAVE_LL_TYPE
+
+typedef NTL_LL_TYPE _ntl_longlong;
+typedef NTL_ULL_TYPE _ntl_ulonglong;
+// typenames are more convenient than macros
+
+#else
+
+#undef NTL_LL_TYPE
+#undef NTL_ULL_TYPE
+// prevent any use of these macros
+
+class _ntl_longlong { private: _ntl_longlong() { } };
+class _ntl_ulonglong { private: _ntl_ulonglong() { } };
+// cannot create variables of these types
+
+
 #endif
 
 /********************************************************/
 
 
+
+// Define an unsigned type with at least 32 bits
+// there is no truly portable way to do this, yet...
+
+
+#if (NTL_BITS_PER_INT >= 32)
+
+typedef unsigned int _ntl_uint32; // 32-bit word
+#define NTL_BITS_PER_INT32 NTL_BITS_PER_INT
+
+#else
+
+// NOTE: C++ standard guarntees longs ar at least 32-bits wide,
+// and this is also explicitly checked at builod time
+
+typedef unsigned long _ntl_uint32; // 32-bit word
+#define NTL_BITS_PER_INT32 NTL_BITS_PER_LONG
+
+#endif
+
+
+
 // The usual token pasting stuff...
 
 #define NTL_PASTE_TOKENS2(a,b) a ## b
