@@ -4,6 +4,11 @@
 
 #include <NTL/ZZ.h>
 #include <NTL/FFT.h>
+#include <NTL/SmartPtr.h>
+#include <NTL/vector.h>
+
+
+
 
 NTL_OPEN_NNS
 
@@ -15,18 +20,23 @@ private:
    void operator=(const zz_pInfoT&); // disabled
 public:
    zz_pInfoT(long NewP, long maxroot);
-   zz_pInfoT(long Index);
-   ~zz_pInfoT();
-
-   long ref_count;
+   zz_pInfoT(INIT_FFT_TYPE, FFTPrimeInfo *info);
+   zz_pInfoT(INIT_USER_FFT_TYPE, long q);
 
    long p;
-   double pinv;
+   mulmod_t pinv;
 
-   long index;        // index >= 0 means we are directly using
-                     // an FFT prime
+   sp_reduce_struct red_struct;
 
-   long PrimeCnt;     // 0 for FFT prime;  otherwise same as NumPrimes
+   FFTPrimeInfo* p_info; // non-null means we are directly using 
+                         // an FFT prime
+
+   UniquePtr<FFTPrimeInfo> p_info_owner;
+   // for user-defined FFT primes, we store the corresponding
+   // FFTPrimeInfo object here
+   
+
+   long PrimeCnt;    // 0 for FFT prime;  otherwise same as NumPrimes
                      // used for establishing crossover points
 
    long NumPrimes;
@@ -34,38 +44,48 @@ public:
    long MaxRoot;
 
    long MinusMModP;  //  -M mod p, M = product of primes
+   mulmod_precon_t MinusMModPpinv;
 
    // the following arrays are indexed 0..NumPrimes-1
    // q = FFTPrime[i]
 
 
-   long *CoeffModP;    // coeff mod p
+   Vec<long> CoeffModP;    // coeff mod p
+   Vec<mulmod_precon_t> CoeffModPpinv; 
 
-   double *x;          // u/q, where u = (M/q)^{-1} mod q
-   long *u;            // u, as above
+   Vec<double> x;               // u/q, where u = (M/q)^{-1} mod q
+   Vec<long> u;                 // u, as above
+   Vec<mulmod_precon_t> uqinv;  // MulModPrecon for u
 };
 
-extern zz_pInfoT *zz_pInfo;  // current modulus, initially null
-
+NTL_THREAD_LOCAL extern SmartPtr<zz_pInfoT> zz_pInfo;  // current modulus, initially null
 
 
 class zz_pContext {
 private:
-zz_pInfoT *ptr;
+SmartPtr<zz_pInfoT> ptr;
 
 public:
+
+zz_pContext() { }
+
+// copy constructor, assignment, destructor: default
+
+explicit zz_pContext(long p, long maxroot=NTL_FFTMaxRoot);
+zz_pContext(INIT_FFT_TYPE, long index);
+zz_pContext(INIT_USER_FFT_TYPE, long q);
+
 void save();
 void restore() const;
 
-zz_pContext() { ptr = 0; }
-zz_pContext(long p, long maxroot=NTL_FFTMaxRoot);
-zz_pContext(INIT_FFT_TYPE, long index);
 
-zz_pContext(const zz_pContext&); 
+// some hooks that are useful in helib...
+// FIXME: generalize these to other context classes
+// and document
 
-zz_pContext& operator=(const zz_pContext&); 
-
-~zz_pContext();
+bool null() const { return ptr == 0; } 
+bool equals(const zz_pContext& other) const { return ptr == other.ptr; } 
+long modulus() const { return ptr->p; }
 
 
 };
@@ -73,8 +93,8 @@ zz_pContext& operator=(const zz_pContext&);
 
 class zz_pBak {
 private:
-long MustRestore;
-zz_pInfoT *ptr;
+zz_pContext c;
+bool MustRestore;
 
 zz_pBak(const zz_pBak&); // disabled
 void operator=(const zz_pBak&); // disabled
@@ -83,32 +103,70 @@ public:
 void save();
 void restore();
 
-zz_pBak() { MustRestore = 0; ptr = 0; }
+zz_pBak() : MustRestore(false) { }
 
 ~zz_pBak();
 
 
 };
 
+
+class zz_pPush {
+private:
+zz_pBak bak;
+
+zz_pPush(const zz_pPush&); // disabled
+void operator=(const zz_pPush&); // disabled
+
+public:
+zz_pPush() { bak.save(); }
+explicit zz_pPush(const zz_pContext& context) { bak.save(); context.restore(); }
+
+explicit zz_pPush(long p, long maxroot=NTL_FFTMaxRoot) 
+   { bak.save(); zz_pContext c(p, maxroot); c.restore(); }
+
+zz_pPush(INIT_FFT_TYPE, long index) 
+   { bak.save(); zz_pContext c(INIT_FFT, index); c.restore(); }
+
+zz_pPush(INIT_USER_FFT_TYPE, long q)
+   { bak.save(); zz_pContext c(INIT_USER_FFT, q); c.restore(); }
+
+};
+
+
+
+
 #define NTL_zz_pRegister(x) zz_p x
 
 
+class zz_pX; // forward declaration
+
 class zz_p {
 public:
+typedef long rep_type;
+typedef zz_pContext context_type;
+typedef zz_pBak bak_type;
+typedef zz_pPush push_type;
+typedef zz_pX poly_type;
+
+
 
 long _zz_p__rep;
 
 
 static void init(long NewP, long maxroot=NTL_FFTMaxRoot);
 static void FFTInit(long index);
+static void UserFFTInit(long q);
 
 
 
 // ****** constructors and assignment
 
-zz_p() { _zz_p__rep = 0; }
+zz_p() : _zz_p__rep(0) {  }
 
-zz_p(const zz_p& a) :  _zz_p__rep(a._zz_p__rep) { }  
+explicit zz_p(long a) : _zz_p__rep(0) { *this = a;  }
+
+zz_p(const zz_p& a) : _zz_p__rep(a._zz_p__rep) { }  
 
 ~zz_p() { } 
 
@@ -121,7 +179,7 @@ long& LoopHole() { return _zz_p__rep; }
 
 static long modulus() { return zz_pInfo->p; }
 static zz_p zero() { return zz_p(); }
-static double ModulusInverse() { return zz_pInfo->pinv; }
+static mulmod_t ModulusInverse() { return zz_pInfo->pinv; }
 static long PrimeCnt() { return zz_pInfo->PrimeCnt; }
 
 
@@ -129,10 +187,25 @@ static long storage() { return sizeof(long); }
 
 zz_p(long a, INIT_LOOP_HOLE_TYPE) { _zz_p__rep = a; }
 
+// for consistency
+zz_p(INIT_NO_ALLOC_TYPE) : _zz_p__rep(0) { } 
+zz_p(INIT_ALLOC_TYPE) : _zz_p__rep(0) { } 
+void allocate() { }
+
+
 };
 
-zz_p to_zz_p(long a);
-void conv(zz_p& x, long a);
+inline
+zz_p to_zz_p(long a) 
+{
+   return zz_p(rem(a, zz_pInfo->p, zz_pInfo->red_struct), INIT_LOOP_HOLE);
+}
+
+inline
+void conv(zz_p& x, long a)
+{
+   x._zz_p__rep = rem(a, zz_pInfo->p, zz_pInfo->red_struct);
+}
 
 inline zz_p& zz_p::operator=(long a) { conv(*this, a); return *this; }
 
@@ -349,6 +422,12 @@ NTL_SNS ostream& operator<<(NTL_SNS ostream& s, zz_p a);
 NTL_SNS istream& operator>>(NTL_SNS istream& s, zz_p& x);
 
 
+void conv(Vec<zz_p>& x, const Vec<ZZ>& a);
+// explicit instantiation of more efficient version,
+// defined in vec_lzz_p.c
+
+
+
 /* additional legacy conversions for v6 conversion regime */
 
 inline void conv(int& x, zz_p a) { conv(x, rep(a)); }
