@@ -6,11 +6,7 @@
 
 NTL_OPEN_NNS
 
-NTL_vector_decl(vec_ulong,vec_vec_ulong)
-
-NTL_eq_vector_decl(vec_ulong,vec_vec_ulong)
-
-NTL_io_vector_decl(vec_ulong,vec_vec_ulong)
+typedef Vec< Vec<unsigned long> > vec_vec_ulong;
 
 NTL_CLOSE_NNS
 
